#!/bin/sh
. ./combine-env.sh

KMAKE_ALL=false
KMAKE_PARALLEL=-j4
KMAKE_REPACK=false

if [ "x$1" = "x--repack" ]; then
  KMAKE_REPACK=true
fi

kmake() {
  NAS_BUILD_TARGET=$1
  echo ""
  echo make ${NAS_BUILD_TARGET}
  fakeroot make KCFLAGS="${KCFLAGS}" \
    KBUILD_BUILD_TIMESTAMP="Debian ${lxdebvb}" KBUILD_BUILD_USER="debian-kernel" KBUILD_BUILD_HOST="lists.debian.org" \
    KBUILD_IMAGE="uImage" KBUILD_BUILD_VERSION=${KBUILD_BUILD_VERSION} ${KMAKE_PARALLEL} -f debian/rules.gen ${NAS_BUILD_TARGET}
  rc=$?
  if [ $rc != 0 ]; then
    fakeroot make KCFLAGS="${KCFLAGS}" \
      KBUILD_BUILD_TIMESTAMP="Debian ${lxdebvb}" KBUILD_BUILD_USER="debian-kernel" KBUILD_BUILD_HOST="lists.debian.org" \
      KBUILD_IMAGE="uImage" KBUILD_BUILD_VERSION=${KBUILD_BUILD_VERSION} -j1 -f debian/rules.gen ${NAS_BUILD_TARGET}
    rc=$?
  fi
  echo Done with return code $rc
  [ $rc != 0 ] && exit $rc
}

mkdir -p archives
cd archives

if [ ! -e ../linux -a ! -e linux-${lxsocka}.tar.xz ]; then
  echo "Downloading linux-${lxsocka}..."
  wget -N ${lxsocua}/linux-${lxsocka}.tar.xz
fi

cd ..

#if [ ! -e config ]; then
#  echo "Downloading config linux-${lxsocka}..."
#  wget -N ${lxsocua}/config
#fi

if [ ! -e linux -a ! -e linux-${lxsocka} ]; then
  echo "Extracting linux-${lxsocka}..."
  tar xJf archives/linux-${lxsocka}.tar.xz
  mv linux-${lxsocka} linux
fi

cd linux/

if [ -e ../linux-nas5xx-3.2.102-git.debian.tar.gz -a ! -e debian ]; then
  tar -xzf ../linux-nas5xx-3.2.102-git.debian.tar.gz
  mv debian/patches .
  quilt push -a
fi

NAS_ARCH=armhf
NAS_MODEL=nas5xx
KERNEL_CONFIG="STG540_Kernel-LT.config"

if [ -e debian/config/${NAS_ARCH}/config.mycloud -o -e arch/arm/configs/sequoia_defconfig ]; then
  NAS_MODEL=mycloud
  KERNEL_CONFIG="MyCloud_Kernel.config"

elif [ -e debian/config/${NAS_ARCH}/config.ds414j -o -e synoconfigs/comcerto2k ]; then
  NAS_MODEL=ds414j
  KERNEL_CONFIG="DS414j_Kernel.config"

elif [ -e debian/config/${NAS_ARCH}/config.tsx31 -o -e include/qnap ]; then
  NAS_MODEL=tsx31
  KERNEL_CONFIG="TSx31_Kernel.config"

elif [ -e arch/arm/configs/gfrg200_defconfig ]; then
  NAS_MODEL=gfrg200
  KERNEL_CONFIG="GFRG200_Kernel.config"
fi

if [ -e arch/arm/mach-kirkwood/nsa325-setup.c ]; then
  NAS_ARCH=armel
  NAS_MODEL=nsa325
fi

VERSION=`cat Makefile | grep -E '^VERSION.=' | cut -d "=" -f 2- | tr -d " \t\n\r"`
PATCHLEVEL=`cat Makefile | grep -E '^PATCHLEVEL.=' | cut -d "=" -f 2- | tr -d " \t\n\r"`
SUBVERSION=`cat Makefile | grep -E '^SUBLEVEL.=' | cut -d "=" -f 2- | tr -d " \t\n\r"`
export ARCH=arm
#export LOCALVERSION="+nas"
LOCALVERSION=
export KBUILD_BUILD_VERSION=${lxsocba}
export KBUILD_DEBARCH=${NAS_ARCH}
export KDEB_PKGVERSION="${VERSION}.${PATCHLEVEL}.${SUBVERSION}-20170609-${NAS_MODEL}"
export KZIP_PKGVERSION="${KDEB_PKGVERSION}"

#export PATH="${PATH}:/opt/OpenWrt-Toolchain-comcerto2000-for-arm_v7-a-gcc-4.5-linaro_glibc-2.14.1_eabi/bin"

export CC=${lxsoccc}

#CROSS="/opt/OpenWrt-Toolchain-comcerto2000-for-arm_v7-a-gcc-4.5-linaro_glibc-2.14.1_eabi/bin/arm-openwrt-linux-"
STAGING_DIR="staging_dir"

echo "creating ${STAGING_DIR} ..."
mkdir ${STAGING_DIR}
export STAGING_DIR="${STAGING_DIR}"

if [ -e debian/config/${NAS_ARCH}/config.${NAS_MODEL} ]; then
  export KDEB_PKGVERSION="${lxdebvb}"
  export KZIP_PKGVERSION="${KDEB_PKGVERSION}-${NAS_MODEL}"
else
  cp -p ../config  ${KERNEL_CONFIG}

  if [ -e arch/arm/configs/sequoia_defconfig ]; then
    cp -p arch/arm/configs/sequoia_defconfig ${KERNEL_CONFIG}
  fi

  cp -av ${KERNEL_CONFIG} .config

  sed -i s/'# CONFIG_EXT4_FS_POSIX_ACL is not set'/'CONFIG_EXT4_FS_POSIX_ACL=y'/g .config

  sed -i s/'# CONFIG_9P_FS_POSIX_ACL is not set'/'CONFIG_9P_FS_POSIX_ACL=y'/g .config

  sed -i s/'# CONFIG_BLK_DEV_NBD is not set'/'CONFIG_BLK_DEV_NBD=m'/g .config
  sed -i s/'# CONFIG_BLK_DEV_RAM is not set'/'CONFIG_BLK_DEV_RAM=m'/g .config
  #sed -i s/'# CONFIG_BLK_DEV_RBD is not set'/'CONFIG_BLK_DEV_RBD=m'/g .config
  #echo 'CONFIG_BLK_DEV_OSD=m' >> .config
  echo 'CONFIG_BLK_DEV_RAM_COUNT=16' >> .config
  echo 'CONFIG_BLK_DEV_RAM_SIZE=4096' >> .config

  cat <<EOFDEBCFG | tee -a .config >/dev/null
CONFIG_NET_SCH_CODEL=m
CONFIG_NET_SCH_FQ_CODEL=m
CONFIG_BPF_JIT=y
# CONFIG_BLK_DEV_XIP is not set
# CONFIG_ALX is not set
CONFIG_AUFS_FS=m
CONFIG_AUFS_BRANCH_MAX_127=y
# CONFIG_AUFS_BRANCH_MAX_511 is not set
# CONFIG_AUFS_BRANCH_MAX_1023 is not set
# CONFIG_AUFS_BRANCH_MAX_32767 is not set
# CONFIG_AUFS_HNOTIFY is not set
CONFIG_AUFS_EXPORT=y
# CONFIG_AUFS_RDU is not set
# CONFIG_AUFS_PROC_MAP is not set
# CONFIG_AUFS_SP_IATTR is not set
# CONFIG_AUFS_SHWH is not set
# CONFIG_AUFS_BR_RAMFS is not set
# CONFIG_AUFS_BR_FUSE is not set
CONFIG_AUFS_BR_HFSPLUS=y
# CONFIG_AUFS_DEBUG is not set
CONFIG_MAGIC_SYSRQ_DEFAULT_MASK=0x01b6
EOFDEBCFG

  #sed -i s/'CONFIG_BLK_DEV_INITRD=.*'/'# CONFIG_BLK_DEV_INITRD is not set'/g .config
  sed -i s/'CONFIG_INITRAMFS_SOURCE=.*'/'CONFIG_INITRAMFS_SOURCE=""'/g .config
  #sed -i s/'CONFIG_INITRAMFS_ROOT_UID=.*'/'# CONFIG_INITRAMFS_ROOT_UID is not set'/g .config
  #sed -i s/'CONFIG_INITRAMFS_ROOT_GID=.*'/'# CONFIG_INITRAMFS_ROOT_GID is not set'/g .config

  if [ "${NAS_MODEL:0:5}" = "nas54" -o "${NAS_MODEL:0:5}" = "nas5x" ]; then
    sed -i s/'# CONFIG_4BAY is not set'/'CONFIG_4BAY=y'/g .config
  elif [ "${NAS_MODEL:0:5}" = "nas52" ]; then
    sed -i s/'CONFIG_4BAY=y'/'# CONFIG_4BAY is not set'/g .config
  fi
fi

if [ "x${NAS_MODEL:0:4}" != "xnsa3" ]; then
  [ ! -e firmware/pmu.elf              ] && cp -p /lib/firmware/pmu.elf      firmware/
  [ ! -e firmware/util_c2000.elf       ] && cp -p /lib/firmware/*_c2000*.elf firmware/
  [ ! -e firmware/util_c2000_revA0.elf ] && cp -p /lib/firmware/util_c2000_revA0.elf firmware/

  #class_c2000.elf
  #[ ! -e firmware/pmu.elf              ] && sed -i s/' pmu.elf'/''/g .config
  #tmu_c2000.elf  util_c2000.elf  
  #[ ! -e firmware/util_c2000_revA0.elf ] && sed -i s/' util_c2000_revA0.elf'/''/g .config
fi

##make HOSTCFLAGS="-O2 -I/home/andy/mindspeed/20140325/sdk-comcerto-openwrt-c2k_nas_3.1-rc2/staging_dir/host/include -Wall -Wmissing-prototypes -Wstrict-prototypes" ARCH=arm CC="arm-openwrt-linux-gnueabi-gcc" CFLAGS="-O3 -pipe -march=armv7-a -mtune=cortex-a9 -fno-caller-saves -mthumb -fhonour-copts  -msoft-float" CROSS_COMPILE=arm-openwrt-linux-gnueabi- KBUILD_HAVE_NLS=no CONFIG_SHELL=/bin/bash oldconfig
#make V=99 HOSTCFLAGS="-O2 -Wall -Wmissing-prototypes -Wstrict-prototypes" ARCH=arm CC="${CROSS}gcc" CFLAGS="-O3 -pipe -march=armv7-a -mtune=cortex-a9 -fno-caller-saves -mthumb -fhonour-copts  -msoft-float" CROSS_COMPILE=${CROSS} KBUILD_HAVE_NLS=no uImage

export KCFLAGS=""
if [ -e debian/config/${NAS_ARCH}/config.tsx31 -o -e include/qnap ]; then
  export KCFLAGS="-DQNAP_HAL=1"
fi

echo "Building ${KZIP_PKGVERSION}..."

if $KMAKE_REPACK ; then
  true
elif [ -e debian/config/${NAS_ARCH}/config.${NAS_MODEL} ]; then
  mkdir -p debian/build

  sed -i s/'gcc-4.6'/${lxsoccc}/g debian/config/defines
  sed -i s/'gcc-4.6'/${lxsoccc}/g debian/config/*/defines

  fakeroot make -f debian/rules debian/control debian/rules.gen

  if $KMAKE_ALL ; then
    for c in `ls debian/config/${NAS_ARCH}/config.${NAS_MODEL:0:3}*`; do
      m=`basename $c`
      m=${m/config./}
      kmake binary-arch_${NAS_ARCH}_none_$m
    done

    kmake binary-arch_${NAS_ARCH}_none
  else
    if [ -e debian/build/build_${NAS_ARCH}_none_${NAS_MODEL} ]; then
      fakeroot make KCFLAGS="${KCFLAGS}" \
        KBUILD_BUILD_TIMESTAMP="Debian ${lxdebvb}" KBUILD_BUILD_USER="debian-kernel" KBUILD_BUILD_HOST="lists.debian.org" \
        KBUILD_IMAGE="uImage" KBUILD_BUILD_VERSION=${KBUILD_BUILD_VERSION} ${KMAKE_PARALLEL} -C debian/build/build_${NAS_ARCH}_none_${NAS_MODEL}
    fi

    kmake binary-arch_${NAS_ARCH}_none_real
    kmake binary-arch_${NAS_ARCH}_none_${NAS_MODEL}
  fi
else
  cp -av .config ${KERNEL_CONFIG}
  cd ..
  ./combine-config.sh ${KERNEL_CONFIG} .config
  cd -
  cp -p .config arch/${ARCH}/configs/${NAS_MODEL}_defconfig

  make ARCH=${ARCH} CC=${CC} KCFLAGS="${KCFLAGS}" ${NAS_MODEL}_defconfig

  make ${KMAKE_PARALLEL} KBUILD_IMAGE="uImage" KBUILD_BUILD_VERSION=${KBUILD_BUILD_VERSION} ARCH=${ARCH} CC=${CC} KCFLAGS="${KCFLAGS}" uImage modules
  make KBUILD_IMAGE="uImage" KBUILD_BUILD_VERSION=${KBUILD_BUILD_VERSION} ARCH=${ARCH} CC=${CC} KCFLAGS="${KCFLAGS}" dtbs

  make KBUILD_IMAGE="uImage" KBUILD_BUILD_VERSION=${KBUILD_BUILD_VERSION} ARCH=${ARCH} CC=${CC} KCFLAGS="${KCFLAGS}" headers_check 
  make KBUILD_IMAGE="uImage" KBUILD_BUILD_VERSION=${KBUILD_BUILD_VERSION} ARCH=${ARCH} CC=${CC} KCFLAGS="${KCFLAGS}" scripts

  make -j1 deb-pkg KDEB_PKGVERSION="${KDEB_PKGVERSION}" ARCH=${ARCH} CC=${CC} KCFLAGS="${KCFLAGS}" \
    KBUILD_IMAGE="uImage" KBUILD_BUILD_VERSION=${KBUILD_BUILD_VERSION} KBUILD_DEBARCH=${KBUILD_DEBARCH} \
    DEBFULLNAME="SCP" \
    DEBEMAIL="admin@nas.local"
  rc=$?

  echo Done with return code $rc
  [ $rc != 0 ] && exit $rc
fi

cd ..

if $KMAKE_REPACK ; then
  true
else
  #sed -i s/'NAS_MODEL=.*'/'NAS_MODEL='${NAS_MODEL}/g build-meta.sh
  sed -i '0,/NAS_MODEL=.*/{s/NAS_MODEL=.*/NAS_MODEL='${NAS_MODEL}'/}' build-meta.sh
  ./build-meta.sh
fi

if [ -e linux/debian/config/${NAS_ARCH}/config.${NAS_MODEL} ]; then
  echo linux-image-${KZIP_PKGVERSION}-${KBUILD_DEBARCH}.zip
  zip linux-image-${KZIP_PKGVERSION}-${KBUILD_DEBARCH}.zip flash-bb-env-and-kernel2.sh install-linux.sh \
    linux-bsp-*_*_*.deb linux-*${NAS_ARCH}_*_*.deb linux-*common*_${KDEB_PKGVERSION}_*.deb linux-*${NAS_MODEL}*_${KDEB_PKGVERSION}_*.deb linux-kbuild-*_*_*.deb *-barebox-boot.tar.gz
else
  echo linux-image-${KZIP_PKGVERSION}-${KBUILD_DEBARCH}.zip
  zip linux-image-${KZIP_PKGVERSION}-${KBUILD_DEBARCH}.zip flash-bb-env-and-kernel2.sh install-linux.sh \
    linux-bsp-*_*_*.deb linux-*_${KDEB_PKGVERSION}_*.deb *-barebox-boot.tar.gz
fi
